import logging
import requests
import json
from .enums import AuthorizationType, HttpMethod
from .errors import UnsupportedAuthorizationType

logger = logging.getLogger(__name__)


class DataService:
    def __init__(
            self,
            base_url,
            auth_type=AuthorizationType.UserCenter,
            username=None,
            password=None,
            jwt_token_company_name=None,
            jwt_token_company_id=None,
            jwt_token_employee_name=None,
            jwt_token_employee_id=None,
            jwt_token_secret=None

    ):
        self.base_url = base_url
        self.auth_type = auth_type

        self.access_token = None
        self.authorization = None

        self.default_header = {
            "Content-Type": "application/json; charset=utf-8",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8"
        }

        self.token = None

        if self.auth_type == AuthorizationType.UserCenter:
            pass
        elif self.auth_type == AuthorizationType.BuiltInUser:
            data, _, _, _ = self.get_auth_token(username, password)
            self.token = data.get('token')
        elif self.auth_type == AuthorizationType.JwtToken:
            pass
        else:
            raise UnsupportedAuthorizationType(self.auth_type)

    def _get_auth_info(self):
        header = self.default_header.copy()

        if self.auth_type == AuthorizationType.UserCenter:
            pass
        elif self.auth_type == AuthorizationType.BuiltInUser:
            header["Authorization"] = self.token
        elif self.auth_type == AuthorizationType.JwtToken:
            header["Jwt-Token"] = self.token
        else:
            raise UnsupportedAuthorizationType(self.auth_type)

        return header

    def _query(self, http_method, api_url, data=None, api_version='v1', if_need_auth=True):
        if if_need_auth:
            header = self._get_auth_info()
        else:
            header = self.default_header.copy()

        url = f"{self.base_url}/api/{api_version}{api_url}"
        if http_method == HttpMethod.GET:
            response = requests.get(url, headers=header)
        elif http_method == HttpMethod.POST:
            response = requests.post(url, headers=header, data=json.dumps(data, ensure_ascii=False).encode('utf-8'))
        else:
            logger.error(f"unsupported request method {http_method}")
            raise Exception(f"unsupported request method {http_method}")

        return response

    @staticmethod
    def _resolve_response(res, if_log_error_when_not_200=True, if_resolve_json_body_when_not_200=False, url=''):
        if res is None:
            logger.error("Response is none")
            raise Exception("Response is none")

        if res.status_code != 200 and if_log_error_when_not_200:
            logger.error(f'Request to [{url}] failed, res status [{res.status_code}], res content [{res.text}]')

        if res.status_code == 200 or if_resolve_json_body_when_not_200:
            data = res.json()
            return data.get("data", None), data.get('message', ''), data.get("code"), data.get('errorCode')

        return None, None, res.status_code, 0

    """Auth"""
    def get_auth_token(self, username, password):
        url = f'/user/auth'
        res = self._query(HttpMethod.POST, url, data={"username": username, "password": password}, if_need_auth=False)
        result, message, status_code, error_code = self._resolve_response(res, if_log_error_when_not_200=True, url=url)
        if status_code != 200:
            logger.warning(
                f'get auth token error, message=[{message}], status_code=[{status_code}],'
                f' error_code={error_code}'
            )
        return result, message, status_code, error_code

    def get_schema(self, schema_api_name, expansion_level=2):
        url = f'/ddl/schema/read/{schema_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def create_schema(self, api_name, display_name, description=''):
        url = f'/ddl/schema/add'
        data = {
            "apiName": api_name,
            "description": description,
            "displayName": display_name,
            "etag": 0,
            "ifEnableCls": False,
            "ifEnableRls": False
        }
        res = self._query(HttpMethod.POST, url, data=data)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def delete_schema(self, schema_api_name, expansion_level=2):
        url = f'/ddl/schema/delete/{schema_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_fields(self, schema_api_name, fields, expansion_level=2):
        url = f'/ddl/schema/{schema_api_name}/field/add?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.POST, url, data=fields)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_index(self, schema_api_name, index, expansion_level=2):
        url = f'/ddl/schema/{schema_api_name}/index/add?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.POST, url, data=index)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_index(self, schema_api_name, index_api_name, expansion_level=2):
        url = f'/ddl/schema/{schema_api_name}/index/read/{index_api_name}?expansionLevel={expansion_level}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def add_records(self, schema_api_name, records, update_or_insert=False, response_with_data=False):
        url = f'/dml/entity/{schema_api_name}/add-batch?upsert={update_or_insert}&responseWithData={response_with_data}'
        res = self._query(HttpMethod.POST, url, data=records)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)

    def get_records(self, schema_api_name, start_id=0, limit=1000, order='asc'):
        url = f'/dml/entity/{schema_api_name}/read?startId={start_id}&limit={limit}&order={order}'
        res = self._query(HttpMethod.GET, url)
        return self._resolve_response(res, if_log_error_when_not_200=True, url=url)


